@@ -244,6 +244,21 @@ void connman_technology_regdom_notify(struct connman_technology *technology,
 	technology->regdom = g_strdup(alpha2);
 }
 
+static int set_regdom_by_device(struct connman_technology *technology,
+							const char *alpha2)
+{
+	GSList *list;
+
+	for (list = technology->device_list; list; list = list->next) {
+		struct connman_device *device = list->data;
+
+		if (connman_device_set_regdom(device, alpha2) != 0)
+			return -ENOTSUP;
+	}
+
+	return 0;
+}
+
 int connman_technology_set_regdom(const char *alpha2)
 {
 	GSList *list;
@@ -251,11 +266,14 @@ int connman_technology_set_regdom(const char *alpha2)
 	for (list = technology_list; list; list = list->next) {
 		struct connman_technology *technology = list->data;
 
-		if (technology->driver == NULL)
-			continue;
+		if (set_regdom_by_device(technology, alpha2) != 0) {
+			if (technology->driver == NULL)
+				continue;
 
-		if (technology->driver->set_regdom)
-			technology->driver->set_regdom(technology, alpha2);
+			if (technology->driver->set_regdom != NULL)
+				technology->driver->set_regdom(technology,
+								alpha2);
+		}
 	}
 
 	return 0;
@@ -723,7 +741,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 		dbus_message_iter_get_basic(&value, &tethering);
 
 		if (technology->tethering == tethering)
-			return __connman_error_in_progress(msg);
+			return __connman_error_already_enabled(msg);
 
 		err = set_tethering(technology, tethering);
 		if (err < 0)
@@ -737,6 +755,9 @@ static DBusMessage *set_property(DBusConnection *conn,
 		if (technology->type != CONNMAN_SERVICE_TYPE_WIFI)
 			return __connman_error_not_supported(msg);
 
+		if (strlen(str) < 1 || strlen(str) > 32)
+			return __connman_error_invalid_arguments(msg);
+
 		technology->tethering_ident = g_strdup(str);
 	} else if (g_str_equal(name, "TetheringPassphrase") == TRUE) {
 		const char *str;
@@ -746,8 +767,8 @@ static DBusMessage *set_property(DBusConnection *conn,
 		if (technology->type != CONNMAN_SERVICE_TYPE_WIFI)
 			return __connman_error_not_supported(msg);
 
-		if (strlen(str) < 8)
-			return __connman_error_invalid_arguments(msg);
+		if (strlen(str) < 8 || strlen(str) > 63)
+			return __connman_error_passphrase_required(msg);
 
 		technology->tethering_passphrase = g_strdup(str);
 	} else if (g_str_equal(name, "Powered") == TRUE) {
@@ -845,6 +866,31 @@ void __connman_technology_scan_stopped(struct connman_device *device)
 		reply_scan_pending(technology, 0);
 }
 
+void __connman_technology_notify_regdom_by_device(struct connman_device *device,
+						int result, const char *alpha2)
+{
+	struct connman_technology *technology;
+	enum connman_service_type type;
+
+	type = __connman_device_get_service_type(device);
+	technology = technology_find(type);
+
+	if (technology == NULL)
+		return;
+
+	if (result < 0) {
+		if (technology->driver != NULL &&
+				technology->driver->set_regdom != NULL) {
+			technology->driver->set_regdom(technology, alpha2);
+			return;
+		}
+
+		alpha2 = NULL;
+	}
+
+	connman_technology_regdom_notify(technology, alpha2);
+}
+
 static DBusMessage *scan(DBusConnection *conn, DBusMessage *msg, void *data)
 {
 	struct connman_technology *technology = data;
